<?php

namespace wbb\data\thread;

use wbb\data\board\BoardCache;
use wbb\data\post\Post;
use wbb\data\post\PostEditor;
use wbb\data\post\ThreadPostList;
use wcf\system\attachment\AttachmentHandler;
use wcf\system\database\util\PreparedStatementConditionBuilder;
use wcf\system\exception\UserInputException;
use wcf\system\message\embedded\object\MessageEmbeddedObjectManager;
use wcf\system\poll\PollManager;
use wcf\system\template\plugin\PagesFunctionTemplatePlugin;
use wcf\system\WCF;

/**
 * @author    Olaf Braun
 * @copyright 2013-2018 Olaf Braun - Software Development
 * @license   WBB-Elite.de License <https://lizenz.wbb-elite.de/lizenz.html>
 * @package   de.wbb-elite.pushXX.board
 * @category  Push++ WBB4.1
 */
class ThreadPushAction extends ThreadAction {
	/**
	 * @var int
	 */
	private $pageNo = 0;
	/**
	 * @var int
	 */
	private $lastPostTime = 0;
	
	/**
	 * @inheritdoc
	 */
	public function getNewPosts() {
		$objectList = new ThreadPostList($this->thread->getDecoratedObject());
		$objectList->sqlLimit = $this->thread->getDecoratedObject()->getBoard()->getPostsPerPage();
		if (!$this->thread->getBoard()->getModeratorPermission('canEnablePost')) $objectList->getConditionBuilder()->add('post.isDisabled = 0');
		if (!$this->thread->getBoard()->getModeratorPermission('canReadDeletedPost')) $objectList->getConditionBuilder()->add('post.isDeleted = 0');
		$objectList->sqlOffset = ($this->pageNo - 1) * $this->thread->getDecoratedObject()->getBoard()->getPostsPerPage();
		$items = $objectList->countObjects();
		$objectList->sqlOrderBy = "post.time, post.postID " . $this->thread->getBoard()->getPostSortOrder();
		$objectList->readObjects();
		$objects = [];
		$startIndex = ($this->pageNo - 1) * $this->thread->getDecoratedObject()->getBoard()->getPostsPerPage();
		$nextPageNotice = '';
		foreach ($objectList->getObjects() as $thread) {
			/** @var ViewableThread $thread */
			if ($thread->time > $this->lastPostTime) {
				$objects[] = $thread;
			}else $startIndex++;
		}
		
		$tpl = new PagesFunctionTemplatePlugin();
		$html = $tpl->execute([
			'print'       => true,
			'object'      => $this->thread->getDecoratedObject(),
			'controller'  => "Thread",
			'application' => "wbb",
			'page'        => $this->pageNo,
			'link'        => "pageNo=%d",
			'pages'       => intval(ceil($items / $this->thread->getDecoratedObject()->getBoard()->getPostsPerPage()))
		], WCF::getTPL());
		
		$conditionBuilder = new PreparedStatementConditionBuilder();
		$conditionBuilder->add('threadID = ?', [$this->thread->threadID]);
		if (!$this->thread->getBoard()->getModeratorPermission('canEnablePost')) $conditionBuilder->add('isDisabled = 0');
		if (!$this->thread->getBoard()->getModeratorPermission('canReadDeletedPost')) $conditionBuilder->add('isDeleted = 0');
		$conditionBuilder->add('time > ?', [$this->lastPostTime]);
		if (count($objectList->getObjectIDs()) > 0) $conditionBuilder->add('postID NOT IN(?)', [$objectList->getObjectIDs()]);
		
		// get post ids
		$sql = "SELECT		postID
				FROM		wbb" . WCF_N . "_post
				" . $conditionBuilder . "
				ORDER BY	TIME, postID " . $this->thread->getBoard()->getPostSortOrder();
		$statement = WCF::getDB()->prepareStatement($sql);
		$statement->execute($conditionBuilder->getParameters());
		$postIDs = $statement->fetchAll(\PDO::FETCH_COLUMN);
		
		if (!empty($postIDs)) {
			$nextPageNotice = WCF::getLanguage()->getDynamicVariable('wbb.thread.newPosts.nextPage', [
				'count'  => count($postIDs),
				'pageNo' => $this->parameters['pageNo'] + 1,
				'thread' => $this->thread->getDecoratedObject()
			]);
		}
		
		$maxPostTime = max($objectList->getMaxPostTime(), $this->lastPostTime);
		
		if (count($objects) == 0) {
			return [
				'lastPostTime'      => $maxPostTime,
				'nextPageNotice'    => $nextPageNotice,
				'template'          => '',
				'templatePageLinks' => $html
			];
		}
		$viewable = new ViewableThread($this->thread->getDecoratedObject());
		// update last visit time count
		if ($viewable->isNew() && $objectList->getMaxPostTime() > $viewable->lastVisitTime) {
			$visitTime = $objectList->getMaxPostTime();
			if ($visitTime == $viewable->getDecoratedObject()->lastPostTime) $visitTime = TIME_NOW;
			$threadAction = new ThreadAction([$viewable->getDecoratedObject()], 'markAsRead', ['visitTime' => $visitTime]);
			$threadAction->executeAction();
		}
		
		WCF::getTPL()->assign([
			'attachmentList' => $objectList->getAttachmentList(),
			'objects'        => $objects,
			'startIndex'     => $startIndex + 1,
			'thread'         => $viewable,
			'sortOrder'      => $this->thread->getBoard()->getPostSortOrder()
		]);
		
		return [
			'lastPostTime'      => $maxPostTime,
			'nextPageNotice'    => $nextPageNotice,
			'template'          => WCF::getTPL()->fetch("threadPostList", "wbb"),
			'templatePageLinks' => $html
		];
	}
	
	/**
	 * @inheritdoc
	 */
	public function validateGetNewPosts() {
		parent::validateGetNewPosts();
		$this->readInteger("lastPostTime", false);
		if (isset($this->parameters["pageNo"])) $this->pageNo = intval($this->parameters["pageNo"]);
		$this->lastPostTime = intval($this->parameters["lastPostTime"]);
	}
	
	/**
	 * Check if the current user can reload this post message
	 *
	 * @throws \wcf\system\exception\PermissionDeniedException
	 */
	public function validateReloadPostMessage() {
		if (isset($this->parameters["pageNo"])) $this->pageNo = intval($this->parameters["pageNo"]);
		$this->readInteger("postID");
		$this->readInteger("index");
		
		$this->thread = $this->getSingleObject();
		if (!$this->thread->canRead()) {
			throw new UserInputException();
		}
	}
	
	/**
	 * Reload the message of a post
	 */
	public function reloadPostMessage() {
		$board = BoardCache::getInstance()->getBoard($this->thread->boardID);
		// load new post
		$post = new Post($this->parameters["postID"]);
		
		$attachmentList = null;
		if (MODULE_ATTACHMENT) {
			// get all attachments except embedded ones
			$attachmentList = $post->getAttachments(true);
			if ($attachmentList !== null) {
				// set permissions
				$attachmentList->setPermissions([
					'canDownload' => $board->getPermission('canDownloadAttachment'),
					'canViewPreview' => $board->getPermission('canViewAttachmentPreview')
				]);
			}
			
			// get total number of attachments
			$attachmentHandler = new AttachmentHandler('com.woltlab.wbb.post', $post->postID);
			$count = count($attachmentHandler);
			
			// update count to reflect number of attachments after edit
			if ($count != $post->attachments) {
				$postEditor = new PostEditor($post);
				$postEditor->update(['attachments' => $count]);
			}
		}
		
		$pollTemplate = '';
		if ($this->thread->canUsePoll(true, $post)) {
			$pollCount = 0;
			$pollID = PollManager::getInstance()->save($post->postID);
			if ($pollID && $pollID != $post->pollID) {
				$postEditor = new PostEditor($post);
				$postEditor->update([
					'pollID' => $pollID
				]);
				
				$pollCount++;
			}
			else if (!$pollID && $post->pollID) {
				$postEditor = new PostEditor($post);
				$postEditor->update([
					'pollID' => null
				]);
				
				$pollCount--;
			}
			
			if ($pollCount != 0) {
				$this->thread->updateCounters(['polls' => $pollCount]);
			}
			
			// reload post
			$post = new Post($post->postID);
			
			if ($post->getPoll()) {
				$pollTemplate = WCF::getTPL()->fetch('poll', 'wcf', ['poll' => $post->getPoll()]);
			}
		}
		
		// load embedded objects
		MessageEmbeddedObjectManager::getInstance()->loadObjects('com.woltlab.wbb.post', [$post->postID]);
		
		$data = [
			'postID' => $post->postID,
			'message' => $post->getFormattedMessage(),
			'notes' => ['wbbPostEditNote' => ''],
			'poll' => $pollTemplate
		];
		
		if ($post->editCount || ($post->getThread()->canEditPost($post) && $post->hasOldVersions())) {
			$data['notes']['wbbPostEditNote'] = WCF::getLanguage()->getDynamicVariable('wbb.post.editNote', [
				'post' => new PostEditor($post),
				'thread' => $post->getThread()
			]);
		}
		
		if (MODULE_ATTACHMENT) {
			WCF::getTPL()->assign([
				'attachmentList' => $attachmentList,
				'objectID' => $post->postID
			]);
			$data['attachmentList'] = WCF::getTPL()->fetch('attachments');
		}
		
		return $data;
	}
}
